/*
 * File:   AnalogJoystick.c
 * Author: PBB
 *
 * Modified on April 28, 2014
 */

#include <stdio.h>
#include <stdlib.h>
#include "configbits32.h"
#include "kit32r7lib.h"
#include <xc.h>
#include "analogHeader.h"
#include <plib.h>

void boardInit(void);
void ocInit(void);

// limit switches:
#define xLimitLeft  PORTDbits.RD6
#define xLimitRight PORTDbits.RD7
#define yLimitDown  PORTDbits.RD8
#define yLimitUp    PORTDbits.RD9
#define xLimitLED   LATDbits.LATD11
#define yLimitLED   LATDbits.LATD10

// motor outputs:
#define yEnablePin      LATEbits.LATE0
#define yDirectionPin   LATEbits.LATE1
#define xEnablePin      LATEbits.LATE2
#define xDirectionPin   LATEbits.LATE3
#define upDownOut       LATEbits.LATE4

// motor inputs:
// Joystick X: AN2/RB2- on kit board this is Port B1 pin 3
// Joystick Y: AN7/RB7 - on kit board this is Port B1 pin 7
#define upDownIn        PORTGbits.RG2

// motor variables
#define FORWARD     0
#define REVERSE     1
#define maxDuty   499
int XstopJoystick, YstopJoystick;
int xDuty, yDuty;
float tempDuty;

int main(int argc, char** argv) {
    float XADCValue, YADCValue;
    int XDirection, YDirection;

    serial_init(57600UL);
    boardInit(); // initialize

    while (1) // repeat continuously
    {
        XADCValue = GetADC(2); // get AN2 value
        YADCValue = GetADC(7); // get AN7 value
        XDirection = XDecide(XADCValue); // decide x direction
        YDirection = YDecide(YADCValue); // decide y direction

        // limit switches
        if (xLimitRight == 0) { // right limit hit
            xLimitLED = 1;
        } else if (xLimitLeft == 0) { // left limit hit
            xLimitLED = 1;
        } else { // limit not hit
            xLimitLED = 0;
        }

        if (yLimitUp == 0) { // up limit hit
            yLimitLED = 1;
        } else if (yLimitDown == 0) { // bottom limit hit
            yLimitLED = 1;
        } else { // limit not hit
            yLimitLED = 0;
        }

        // up/down
        if (upDownIn == 0) { // up/down button pushed
            upDownOut = 0;
        } else {
            upDownOut = 1;
        }

        // X direction
        if (XDirection == 1) { // Go Right
            XstopJoystick = 0;
            tempDuty = ((maxDuty+2)/(493))*(XADCValue-530);
            xDuty = (int) tempDuty;
            if (xDuty > (maxDuty + 2)) {
                xDuty = (maxDuty + 2); // 100% duty cycle: fastest for forward
            }
            OC5RS = xDuty; // load OC1RS with new duty cycle
            xDirectionPin = FORWARD;
            if (xLimitRight == 1) {
                xEnablePin = 1;
            } else { // limit has been hit. Don't move right
                xEnablePin = 0;
            }
        } else if (XDirection == 2) { // Go Left
            XstopJoystick = 0;
            tempDuty = (maxDuty+2)/(493)*(XADCValue);
            xDuty = (int) tempDuty;
            if (xDuty < 0) {
                xDuty = 0; // 0% duty cycle: fastest for reverse
            }
            OC5RS = xDuty; // load OC1RS with new duty cycle
            xDirectionPin = REVERSE;
            if (xLimitLeft == 1) {
                xEnablePin = 1;
            } else { // limit has been hit. Don't move left
                xEnablePin = 0;
            }
        } else if ((XDirection == 0) && (XstopJoystick == 0)) { // stop
            XstopJoystick = 1;
            xDuty = 0;
            OC5RS = xDuty; // turn off motor
            xDirectionPin = 0;
            xEnablePin = 0;
        } else {
            // do nothing
        }

        // y direction
        if (YDirection == 1) { // Go Up
            YstopJoystick = 0;
            tempDuty = 0.659229*YADCValue-349.39148;
            yDuty = (int) tempDuty;
            if (yDuty > (maxDuty + 2)) {
                yDuty = (maxDuty + 2); // 100% duty cycle: fastest for forward
            }
            OC1RS = yDuty; // load OC5RS with new duty cycle
            yDirectionPin = FORWARD;
            if (yLimitUp == 1) {
                yEnablePin = 1;
            } else { // limit has been hit. Don't move up
                yEnablePin = 0;
            }
        } else if (YDirection == 2) { // Go Down
            YstopJoystick = 0;
            tempDuty = 0.659229*YADCValue + 176;
            yDuty = (int) tempDuty;
            if (yDuty < 0) {
                yDuty = 0; // 0% duty cycle: fastest for reverse
            }
            OC1RS = yDuty; // load OC5RS with new duty cycle
            yDirectionPin = REVERSE;
            if (yLimitDown == 1) {
                yEnablePin = 1;
            } else { // limit has been hit. Don't move down
                yEnablePin = 0;
            }
        } else if ((YDirection == 0) && (YstopJoystick == 0)) { // stop
            YstopJoystick = 1;
            yDuty = 0;
            OC1RS = yDuty; // turn off motor
            yDirectionPin = 0;
            yEnablePin = 0;
        } else {
            // do nothing
        }

    } // repeat

    CloseOC1();
    CloseOC5();
    return (EXIT_SUCCESS);
}

void boardInit() {
    TRISE = 0b11111111; // everything is input for now
    TRISB = 0b1111111111111111; // everything is input for now
    TRISD = 0b11111111; // everything is input for now
    TRISG = 0b111111; // everything is input for now

    // set outputs
    TRISEbits.TRISE0 = 0;  // RE0 (X Enable Pin is Output)
    TRISEbits.TRISE1 = 0;  // RE1 (X Direction Pin is Output)
    TRISEbits.TRISE2 = 0;  // RE2 (Y Enable Pin is Output)
    TRISEbits.TRISE3 = 0;  // RE3 (Y Direction Pin is Output)
    TRISEbits.TRISE4 = 0;  // RE4 (Up/Down Pin is Output)
    TRISDbits.TRISD10 = 0; // RD10 (X Limit LED Pin is Output)
    TRISDbits.TRISD11 = 0; // RD11 (Y Limit LED Pin is Output)

    // initialize variables
    XstopJoystick = 1; // no motion initially
    YstopJoystick = 1; //no motion initially
    xDuty = 0;
    yDuty = 0;

    yLimitLED = 0;
    xLimitLED = 0;
    upDownOut = 1;

    // ADC Initialize
    AD1CON1 = 0x0000; // SAMP bit = 0 ends sampling and starts converting
    AD1CSSL = 0;
    AD1CON3 = 0x0002; // Manual Sample, TAD = internal 6 TPB
    AD1CON2 = 0;

    xDirectionPin = FORWARD;
    xEnablePin = 0;

    yDirectionPin = FORWARD;
    yEnablePin = 0;

    // initalize OC and timers
    ocInit();

    asm("ei"); // enable interrupts
    INTCONbits.MVEC = 1; // enable multivector interrupts

    // set up for Timer 2 interrupt
    INTEnable(INT_T2, INT_ENABLED);
    INTSetVectorPriority(INT_T2, 6);

}

void ocInit(void) {
    // Timer2 Setup
    T2CON = 0x0000; // Stop timer and clear control register
    T2CONbits.TCKPS = 0; // set prescaler at 1:1
    TMR2 = 0x0000; // Clear the timer register
    PR2 = maxDuty; // Load period register
    T2CONbits.ON = 1; // Start Timer

    // OC1 Setup
    OC1CON = 0x0000; // Turn off the OC1 when performing the setup
    OC1R = 0; // Initialize primary compare register to 0 duty cycle
    OC1RS = 0; // Initialize secondary compare register to 0 duty cycle
    OC1CONbits.OCM = 0b110; // PWM mode on; Fault pin disabled
    OC1CONbits.OCTSEL = 0; // Timer2 is the clock source for this output compare module
    OC1CONbits.ON = 1; // enable OC1

    // OC5 Setup
    OC5CON = 0x0000; // Turn off the OC1 when performing the setup
    OC5R = 0; // Initialize primary compare register to 50% duty cycle
    OC5RS = 0; // Initialize secondary compare register to 50% duty cycle
    OC5CONbits.OCM = 0b110; // PWM mode on; Fault pin disabled
    OC5CONbits.OCTSEL = 0; // Timer2 is the clock source for this output compare module
    OC5CONbits.ON = 1; // enable OC5

}

void __ISR(_TIMER_2_VECTOR, ipl6) T2Interrupt(void) {
    IFS0bits.T2IF = 0; // clear interrupt flag and exit
}